/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.registry.core.gms;

import java.util.HashSet;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.petals.registry.api.LocalRegistry;
import org.ow2.petals.registry.api.RemoteRegistry;
import org.ow2.petals.registry.api.context.Context;
import org.ow2.petals.registry.api.exception.LifeCycleException;
import org.ow2.petals.registry.api.lifecycle.AbstractLifeCycle;

/**
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 */
public class GMSManagerImpl extends AbstractLifeCycle implements GMSManager {

    /**
     * The class logger
     */
    private Log log = LogFactory.getLog(GMSManagerImpl.class);
    
    private LocalRegistry localRegistry;
    
    /**
     * Send hearbeat to alive members
     * 
     */
    private class HeartBeatTask extends TimerTask {
        @Override
        public void run() {
            for (RemoteRegistry remoteRegistry : liveRegistries) {
                if (log.isInfoEnabled()) {
                    log.info("HB to " + remoteRegistry.getURI());
                    heartbeat();
                }
            }
        }
    }

    /**
     * Send heartbeat to dead members
     * 
     */
    @SuppressWarnings("unused")
    private class UnreachableTask extends TimerTask {
        @Override
        public void run() {
            for (RemoteRegistry remoteRegistry : unreachableRegistries) {
                if (log.isInfoEnabled()) {
                    log.info("HB to unreachable " + remoteRegistry.getURI());
                    heartbeat();
                }
            }
        }
    }

    private Timer hbTimer;

    private Timer deadTimer;

    /* live member set */
    private Set<RemoteRegistry> liveRegistries = new HashSet<RemoteRegistry>();

    /* unreachable member set */
    private Set<RemoteRegistry> unreachableRegistries = new HashSet<RemoteRegistry>();

    public static int DELAY = 10000;

    /**
     * Creates a new instance of GMSManagerImpl
     * 
     * @param localRegistry
     */
    public GMSManagerImpl(LocalRegistry localRegistry) {
        this.localRegistry = localRegistry;
    }

    @Override
    protected void doInit(Context context) throws LifeCycleException {
        hbTimer = new Timer(false);
        deadTimer = new Timer(false);
    }

    @Override
    protected void doStart() throws LifeCycleException {
        hbTimer.schedule(new HeartBeatTask(), DELAY, DELAY);
        deadTimer.schedule(new HeartBeatTask(), DELAY, DELAY);
    }

    @Override
    protected void doStop() throws LifeCycleException {
        this.hbTimer.cancel();
        this.deadTimer.cancel();
    }

    public void join(Group group) {
    }

    public void heartbeat() {
    }

    public void quit(Group group) {
        // TODO Auto-generated method stub
    }
    
    public LocalRegistry getLocalRegistry() {
        return localRegistry;
    }
}
