/*
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.registry.core.repository.bo;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.transform.TransformerException;

//import org.ow2.petals.registry.api.Endpoint;
import org.ow2.petals.registry.api.Resource;
import org.ow2.petals.registry.api.util.XMLUtil;

/**
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 */
public class TOAdapterImpl implements TOAdapter {

    public Resource toTO(org.ow2.petals.registry.core.repository.bo.Endpoint boEndpoint) {
        Resource result = new Resource();
        org.ow2.petals.registry.api.Endpoint resourceEndpoint = new org.ow2.petals.registry.api.Endpoint();
        
        if (boEndpoint == null) {
            return null;
        }

        result.setCreationTime(boEndpoint.getCreationTime());
        result.setEndpoint(resourceEndpoint);

        if (boEndpoint.getPath() != null) {
            result.setPath(boEndpoint.getPath());
        }

        if (boEndpoint.getRootRegistry() != null) {
            result.setRootRegistry(boEndpoint.getRootRegistry());
        }

        if (boEndpoint.getDescription() != null) {
            resourceEndpoint.setDescription(XMLUtil.createDocumentFromString(boEndpoint.getDescription()));
        }

        if (boEndpoint.getName() != null) {
            resourceEndpoint.setName(QName.valueOf(boEndpoint.getName()));
        }

        if (boEndpoint.getService() != null) {
            resourceEndpoint.setService(QName.valueOf(boEndpoint.getService()));
        }

        if (boEndpoint.getInterfaceName() != null) {
            List<QName> interfaces = new ArrayList<QName>();         
            Interface itf = boEndpoint.getInterfaceName();
            
            interfaces.add(QName.valueOf(itf.getName()));
            
            resourceEndpoint.setInterfaces(interfaces);
        }

        if ((boEndpoint.getProperties() != null) && (boEndpoint.getProperties().getValues() != null)) {
            for (Property p : boEndpoint.getProperties().getValues()) {
                resourceEndpoint.getProperties().put(p.getClef(), p.getValue());
            }
        }
        
        return result;
    }

    public org.ow2.petals.registry.core.repository.bo.Endpoint toBO(Resource resource) {
        org.ow2.petals.registry.core.repository.bo.Endpoint result = new org.ow2.petals.registry.core.repository.bo.Endpoint();
        org.ow2.petals.registry.api.Endpoint resourceEndpoint = null;

        if (resource.getEndpoint() == null) {
            return null;
        }

        resourceEndpoint = resource.getEndpoint();

        result.setCreationTime(resource.getCreationTime());
        result.setPath(resource.getPath());
        result.setRootRegistry(resource.getRootRegistry());

        try {
            if (resourceEndpoint.getDescription() != null) {
                result.setDescription(XMLUtil
                        .createStringFromDOMDocument(resourceEndpoint.getDescription()));
            }
        } catch (TransformerException e) {
            e.printStackTrace();
        }

        if (resourceEndpoint.getName() != null) {
            result.setName(resourceEndpoint.getName().toString());
        }

        if (resourceEndpoint.getService() != null) {
            result.setService(resourceEndpoint.getService().toString());
        }

        if (resourceEndpoint.getInterfaces() != null) {
          //  List<Interface> l = new ArrayList<Interface>();
            Interface it = new Interface ();
            if(! resourceEndpoint.getInterfaces().isEmpty()){
                QName qn = resourceEndpoint.getInterfaces().get(0);
                it.setName(qn.toString());
            }
         
            result.setInterfaceName(it);
            
          
        }

        //Properties of the resource
        if (resource.getProperties() != null) {
            List<Property> properties = new ArrayList<Property>();
            Iterator<String> iter = resource.getProperties().keySet().iterator();
            while (iter.hasNext()) {
                String key = iter.next();
                Property p = new Property();
                p.setClef(key);
                p.setValue(resource.getProperties().get(key));
                properties.add(p);
            }
            Properties props = new Properties();
            props.setValues(properties);
            result.setProperties(props);
        }

        //Properties of the endpoint
        if (resourceEndpoint.getProperties() != null) {
            List<Property> properties = new ArrayList<Property>();
            Iterator<String> iter = resourceEndpoint.getProperties().keySet().iterator();
            while (iter.hasNext()) {
                String key = iter.next();
                Property p = new Property();
                p.setClef(key);
                p.setValue(resourceEndpoint.getProperties().get(key));
                properties.add(p);
            }
            Properties props = new Properties();
            props.setValues(properties);
            result.setProperties(props);
        }
        
        return result;
    }
 

}
