/*
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.registry.core.repository.dao;

import java.util.Iterator;
import java.util.List;

import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.ow2.petals.registry.core.repository.bo.Endpoint;
import org.ow2.petals.registry.core.repository.bo.Interface;
import org.ow2.petals.registry.core.repository.bo.Properties;
import org.ow2.petals.registry.core.repository.hibernate.HibernateHelper;

import com.trg.search.Filter;
import com.trg.search.Search;
import com.trg.search.hibernate.HibernateSearchProcessor;

/**
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 *         public Object create(Object object) { Session session = null; try {
 *         session = sessionFactory.openSession(); Transaction tx =
 *         session.beginTransaction(); session.save(object); session.flush();
 *         tx.commit(); ... }
 * 
 */
public class EndpointDAO //extends GenericDAO<Endpoint> 
{

    private static Log logger = LogFactory.getLog(EndpointDAO.class);

    public EndpointDAO() {
        super();
    }

    public boolean endpointExists(Session session, String path) throws DAOException {
        return this.get(session, path) != null;
    }

    public Endpoint get(Session session, String path) throws DAOException {
        Endpoint result = null;
        logger.debug("Get resource from " + path);
        final String query = "from Endpoint edp WHERE edp.path = '" + path + "'";

        try {
            Query q = session.createQuery(query);
            //q.setParameter("path", path);    
            result = (Endpoint) q.uniqueResult();
        } catch (HibernateException e) {

            logger.debug("Error in the method get of EndpointDAO ", e);
            throw new DAOException(e.getMessage());
        }
        return result;
    }

    public List<Endpoint> getEndpointPath(Session session, String path) throws DAOException {
        List<Endpoint> result = null;

        logger.debug("*******************/////////////////////////////Get resources from " + path + "****************************////////////////////");
        final String query = "from Endpoint edp WHERE edp.path like '" + path + "%'";

        try {
            Query q = session.createQuery(query);
            //q.setParameter("path", path);    
            result = q.list();
        } catch (HibernateException e) {

            logger.debug("Error in the method get of EndpointDAO ", e);
            throw new DAOException(e.getMessage());
        }
        return result;
    }

    public Interface getInter(Session session, Endpoint endpoint) throws DAOException {
        Interface it = null;

        String itName = endpoint.getInterfaceName().getName();
        String query = "from Interface it where it.name = '" + itName + "'";

        try {
            Query q = session.createQuery(query);
            it = (Interface) q.uniqueResult();

        } catch (HibernateException e) {
            logger.debug("Error in the method getInter of EndpointDAO ", e);
            throw new DAOException(e.getMessage());
        }

        return it;
    }

    public Properties getProperties(Session session, Endpoint endpoint) throws DAOException {

        Properties pr = null;

        String path = endpoint.getPath();
        String query = "from Properties pr where pr.pathEnd='" + path + "'";

        try {
            Query q = session.createQuery(query);
            pr = (Properties) q.uniqueResult();
        } catch (HibernateException e) {

            logger.debug("Error in the method getInter of EndpointDAO ", e);
            throw new DAOException(e.getMessage());
        }
        return pr;
    }

    public String getId(Session session, String path) {
        String result = null;
        String query = "from Endpoint edp WHERE edp.path = :path";
        Query q = session.createQuery(query);
        q.setString("path", path);
        Endpoint e = (Endpoint) q.uniqueResult();
        if (e != null) {
            result = e.getId();
        }
        return result;
    }

    public void update(Session session, Endpoint endpoint, String path) throws DAOException {
        // get the id if it exists
        // TODO : Thing about transaction...

        String id = this.getId(session, path);
        if (id != null) {
            endpoint.setId(id);
            this.merge(session, endpoint);
        } else {
            throw new DAOException("Can not find intial resource to update");
        }

    }

    public void merge(Session session, Endpoint endpoint) throws DAOException {
        Transaction tx = null;
        Interface it;
        Properties pr;

        try {
            tx = session.beginTransaction();

            it = this.getInter(session, endpoint);
            if (it != null) {
                endpoint.setInterfaceName(it);
            } else if (endpoint.getInterfaceName().getName() != null) {
                this.createInterface(session, endpoint);
                it = this.getInter(session, endpoint);
                endpoint.setInterfaceName(it);
            } else {
                it = null;
                endpoint.setInterfaceName(it);
            }

            pr = this.getProperties(session, endpoint);
            if (pr != null) {
                endpoint.setProperties(pr);
            } else if (!endpoint.getProperties().getValues().isEmpty()) {
                this.createProperties(session, endpoint);
                pr = this.getProperties(session, endpoint);
                endpoint.setProperties(pr);
            } else {
                pr = null;
                endpoint.setProperties(pr);
            }

            session.merge(endpoint);

            tx.commit();
        } catch (final Exception e) {
            if (tx != null) {
                tx.rollback();
            }
            if (endpoint != null) {
                throw new DAOException("Error merging " + endpoint.getClass().getName(), e);
            }
            throw new DAOException("Error merging Object of type null", e);
        }
    }

    public void create(Session session, Endpoint endpoint) throws DAOException {

        Transaction tx = null;
        Interface it;
        Properties pr = endpoint.getProperties();
        try {
            tx = session.beginTransaction();

            it = this.getInter(session, endpoint);
            if (it != null) {
                endpoint.setInterfaceName(it);
            } else if (endpoint.getInterfaceName().getName() != null) {
                this.createInterface(session, endpoint);
                it = this.getInter(session, endpoint);
                endpoint.setInterfaceName(it);
            } else {
                it = null;
                endpoint.setInterfaceName(it);
            }

            pr = this.getProperties(session, endpoint);
            if (pr != null) {
                endpoint.setProperties(pr);
            } else if (!endpoint.getProperties().getValues().isEmpty()) {
                this.createProperties(session, endpoint);
                pr = this.getProperties(session, endpoint);
                endpoint.setProperties(pr);
            } else {
                pr = null;
                endpoint.setProperties(pr);
            }

            session.save(endpoint);
            tx.commit();
        } catch (final HibernateException e) {
            if (tx != null) {
                tx.rollback();
            }
            if (endpoint != null) {
                throw new DAOException("Error creating new " + endpoint.getClass().getName(), e);

            }
            throw new DAOException("Error creating new Object of type null", e);
        }
    }

    public void createInterface(Session session, Endpoint endpoint) throws DAOException {
        Interface it = new Interface();
        it.setName(endpoint.getInterfaceName().getName());
        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            session.save(it);
        } catch (final Exception e) {
            if (tx != null) {
                tx.rollback();
            }
            if (endpoint != null) {
                throw new DAOException("Error creating new " + endpoint.getClass().getName(), e);
            }
            throw new DAOException("Error creating new Object of type null", e);
        }
    }

    public void createProperties(Session session, Endpoint endpoint) throws DAOException {

        Properties pr = new Properties();
        pr = endpoint.getProperties();
        pr.setPathEnd(endpoint.getPath());
        Transaction tx = null;
        try {
            tx = session.beginTransaction();

            session.save(pr);
        } catch (final Exception e) {
            if (tx != null) {
                tx.rollback();
            }
            if (endpoint != null) {
                throw new DAOException("Error creating new " + endpoint.getClass().getName(), e);
            }
            throw new DAOException("Error creating new Object of type null", e);
        }
    }

    public boolean delete(Session session, String path) throws DAOException {
        boolean result = true;

        if (logger.isDebugEnabled()) {
            logger.debug("Delete the resource from " + path);
        }

        if ((path == null) || (path.trim().length() == 0)) {
            if (logger.isDebugEnabled()) {
                logger.debug("Can not delete an entry under null or empty path");
            }
            result = false;
        } else {
            if (path.endsWith("/")) {
                this.deletePath(session, path);
            } else {
                this.deleteEntry(session, path);
            }
        }
        return result;
    }

    @SuppressWarnings("unchecked")
    public void deleteInterface(Session session, String id) throws DAOException {

        String queryEnd = "select * from Endpoint ed where ed.interface_id = '" + id + "'";
        //  String query = "select * from Interface it where it.id ='"+id+"'";
        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            Query q = session.createSQLQuery(queryEnd).addEntity(Endpoint.class);
            List<Endpoint> listEnd = q.list();

            if (listEnd == null || listEnd.isEmpty()) {
                Interface it = (Interface) session.load(Interface.class, id);
                session.delete(it);
            }
            tx.commit();
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException("Error !!! to delete Interface", e2);
                }
            }
            throw new DAOException(e.getMessage());
        }
    }

    public void deleteProperties(Session session, String id) throws DAOException {


        String query = "delete from Properties pr where pr.id = '" + id + "'";
        String queryPropersProper = "delete from Properties_Property where properties_id= '" + id + "'";
        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            Query q2 = session.createSQLQuery(queryPropersProper);
            q2.executeUpdate();

            Query q = session.createQuery(query);
            q.executeUpdate();
            tx.commit();
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }
    }

    public int deletePath(final Session session, final String path) throws DAOException {
        int result = 0;
        final String query = "delete from Endpoint ed WHERE ed.path like '" + path + "%'";
        List<Endpoint> endpoint = this.getEndpointPath(session, path);
        Query q = session.createQuery(query);
        logger.debug("Query =" + query);
        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            result = q.executeUpdate();
            tx.commit();
            logger.debug(result + " entries have been deleted");
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }

        if (endpoint != null) {
            for (Endpoint edp : endpoint) {
                if (edp.getInterfaceName() != null) {
                    this.deleteInterface(session, edp.getInterfaceName().getId());
                }
                if (edp.getProperties() != null) {
                    this.deleteProperties(session, edp.getProperties().getId());
                }
            }
        }

        return result;
    }

    public boolean deleteEntry(final Session session, final String path) throws DAOException {
        boolean result = true;
        logger.debug("Delete the entry from " + path);

        // get then delete...
        Transaction tx = null;
        try {
            tx = session.beginTransaction();

            Endpoint ed = this.get(session, path);

            if (ed != null) {
                session.delete(ed);
                session.flush();
            }
            tx.commit();

            if (ed.getInterfaceName() != null) {
                this.deleteInterface(session, ed.getInterfaceName().getId());
            }
            if (ed.getProperties() != null) {

                this.deleteProperties(session, ed.getProperties().getId());
            }

        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());

                }
            }
            throw new DAOException(e.getMessage());
        }



        return result;
    }

    public int deleteAll(Session session) throws DAOException {
        int result = 0;

        final String query = "delete from Endpoint";
        String queryInter = "delete from Interface";
        String queryPropers = "delete from Properties";

        String queryPropersProper = "delete from Properties_Property";
        String queryProper = "delete from Property";
        logger.debug("Query =" + query);
        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            Query q = session.createQuery(query);
            result = q.executeUpdate();

            q = session.createQuery(queryInter);
            q.executeUpdate();
            Query q2 = session.createSQLQuery(queryPropersProper);
            q2.executeUpdate();

            q = session.createQuery(queryProper);
            q.executeUpdate();

            q = session.createQuery(queryPropers);
            q.executeUpdate();
            tx.commit();
            logger.debug(result + " entries have been deleted");
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }
        return result;
    }

    public int deleteFrom(Session session, String registry) throws DAOException {
        int result = 0;

        List<Endpoint> endpoint = this.getAllFromRegistry(session, registry);

        final String query = "delete from Endpoint where rootRegistry = :rootRegistry";
        Query q = session.createQuery(query);
        q.setString("rootRegistry", registry);
        logger.debug("Query =" + query);
        Transaction tx = null;


        try {

            tx = session.beginTransaction();
            result = q.executeUpdate();
            tx.commit();
            logger.debug(result + " entries have been deleted");
            for (int i = 0; i < endpoint.size(); i++) {
                this.deleteInterface(session, endpoint.get(i).getInterfaceName().getId());
                this.deleteProperties(session, endpoint.get(i).getProperties().getId());
            }
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }



        return result;
    }

    @SuppressWarnings("unchecked")
    public List<Endpoint> getAllFromRegistry(Session session, String registry) throws DAOException {

        List<Endpoint> endpoint = null;
        String query = "from Endpoint ed where ed.rootRegistry !='" + registry + "'";
        try {
            Query q = session.createQuery(query);
            endpoint = q.list();
        } catch (HibernateException e) {

            logger.debug("Error in the method get of EndpointDAO ", e);
            throw new DAOException(e.getMessage());
        }


        return endpoint;
    }

    @SuppressWarnings("unchecked")
    public List<Endpoint> getAllFromPath(Session session, String path) throws DAOException {

        List<Endpoint> list = null;

        logger.debug("Get all the endpoints under : " + path);
        final String query = "from Endpoint ed WHERE ed.path like '" + path + "%'";
        logger.debug("Query = " + query);

        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            list = session.createQuery(query).list();

            tx.commit();
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }

        return list;
    }

    @SuppressWarnings("unchecked")
    public List<Endpoint> query(Session session, org.ow2.petals.registry.api.Query query)
            throws DAOException {

        List<Endpoint> list = null;
        try {
            //List<Resource> list = null;
            logger.debug("Query " + query);

            HibernateSearchProcessor processor = HibernateSearchProcessor.getInstanceForSessionFactory(HibernateHelper.getSessionFactory());
            Search search = new Search();
            search.setSearchClass(Endpoint.class);
            search.setDistinct(true);
            if (query.getEndpoint() != null) {
                search.addFilter(Filter.equal("name", query.getEndpoint().toString()));
            }

            if (query.getService() != null) {
                search.addFilterEqual("service", query.getService().toString());
            }

            if ((query.getInterfaces() != null) && (query.getInterfaces().size() > 0)) {
                Filter and = Filter.and();

                if (query.getInterfaces() != null) {
                    QName s = query.getInterfaces().get(0);
                    and.add(Filter.some("interfaceName", Filter.equal("name", s.toString())));
                    search.addFilter(and);
                }
            }


            if (!query.isIgnoreParams() && (query.getParams() != null) && (query.getParams().size() > 0)) {
                logger.debug("Using query parameters");
                Filter properties = Filter.and();
                Iterator<String> iter = query.getParams().keySet().iterator();
                while (iter.hasNext()) {
                    String key = iter.next();
                    Filter f = Filter.some("properties.values", Filter.and(Filter.equal("clef",
                            key), Filter.equal("value", query.getParams().get(key))));
                    properties.add(f);
                }
                search.addFilter(properties);
            }

            //System.out.println(search.toString());
            list = processor.search(session, search);
        } catch (RuntimeException ex) {
            ex.printStackTrace();
            logger.debug("wooops");
        }

        return list;
    }

    public long count(Session session, Class<Endpoint> endpoint) throws DAOException {
        try {
            final Query qr = session.createQuery("select count(*) from " + endpoint.getName());
            if (qr != null) {
                return ((Long) qr.uniqueResult()).longValue();
            } else {
                return -1;
            }
        } catch (final Exception e) {
            if (endpoint != null) {
                throw new DAOException("Error getting size " + endpoint.getName(), e);
            }
            throw new DAOException("Error getting size of type null", e);
        }
    }

    public int keepOnly(Session session, String registry) throws DAOException {
        int result = 0;

        List<Endpoint> endpoint = this.getAllFromRegistry(session, registry);

        final String query = "delete from Endpoint where rootRegistry != :rootRegistry";
        Query q = session.createQuery(query);
        q.setString("rootRegistry", registry);

        if (logger.isDebugEnabled()) {
            logger.debug("Query = '" + query + "' and rootregistry is " + registry);
        }

        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            result = q.executeUpdate();
            tx.commit();

            if (endpoint != null && !endpoint.isEmpty()) {
                for (int i = 0; i < endpoint.size(); i++) {
                    if (endpoint.get(i).getInterfaceName() != null) {
                        this.deleteInterface(session, endpoint.get(i).getInterfaceName().getId());
                    }
                    if (endpoint.get(i).getProperties() != null) {
                        this.deleteProperties(session, endpoint.get(i).getProperties().getId());
                    }
                }
            }
            if (logger.isDebugEnabled()) {
                logger.debug(result + " entries have been deleted");
            }
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }
        return result;
    }
}
