/*
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.registry.core.repository.dao;

import java.util.Iterator;
import java.util.List;

import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.ow2.petals.registry.core.repository.bo.Resource;
import org.ow2.petals.registry.core.repository.hibernate.HibernateHelper;

import com.trg.search.Filter;
import com.trg.search.Search;
import com.trg.search.hibernate.HibernateSearchProcessor;

/**
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 */
public class ResourceDAO extends GenericDAO<Resource> {

    private static Log logger = LogFactory.getLog(ResourceDAO.class);

    public ResourceDAO() {
        super();
    }

    /**
     * TODO : Optimize to not load the object but just get it from an HQL query.
     * 
     * @param session
     * @param path
     * @return
     * @throws DAOException
     */
    public boolean resourceExists(Session session, String path) throws DAOException {
        return this.get(session, path) != null;
    }

    /**
     * Get all the resources under the given path
     * 
     * @param session
     * @param path
     * @return
     * @throws DAOException
     */
    @SuppressWarnings("unchecked")
    public List<Resource> getAllFromPath(Session session, String path) throws DAOException {
        List<Resource> list = null;
        logger.debug("Get all the resources under : " + path);
        final String query = "from Resource res WHERE res.path like '" + path + "%'";
        logger.debug("Query = " + query);
        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            list = session.createQuery(query).list();
            tx.commit();
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }

        return list;
    }

    public List<Resource> query(Session session, org.ow2.petals.registry.api.Query query)
            throws DAOException {

        List<Resource> list = null;
        try {
        //List<Resource> list = null;
        logger.debug("Query " + query);

        HibernateSearchProcessor processor = HibernateSearchProcessor
                .getInstanceForSessionFactory(HibernateHelper.getSessionFactory());
        Search search = new Search();
        search.setSearchClass(Resource.class);
        search.setDistinct(true);
        if (query.getEndpoint() != null) {
            search.addFilter(Filter.equal("endpoint.name", query.getEndpoint().toString()));
        }

        if (query.getService() != null) {
            search.addFilterEqual("endpoint.service", query.getService().toString());
        }

        if ((query.getInterfaces() != null) && (query.getInterfaces().size() > 0)) {
            Filter and = Filter.and();
            for (QName s : query.getInterfaces()) {
                and.add(Filter.some("endpoint.interfaces", Filter.equal("name", s.toString())));
            }
            search.addFilter(and);
        }

        if (!query.isIgnoreParams() && (query.getParams() != null)
                && (query.getParams().size() > 0)) {
            logger.debug("Using query parameters");
            Filter properties = Filter.and();
            Iterator<String> iter = query.getParams().keySet().iterator();
            while (iter.hasNext()) {
                String key = iter.next();
                Filter f = Filter.some("endpoint.properties.values", Filter.and(Filter.equal("clef",
                        key), Filter.equal("value", query.getParams().get(key))));
                properties.add(f);
            }
            search.addFilter(properties);
        }
        list = processor.search(session, search);
    } catch (RuntimeException ex) {
        this.logger.debug("wooops");
    }

        return list;
    }

    /**
     * Get the unique resource under the given path.
     * 
     * @param session
     * @param path
     *            a resource path (not a folder)
     * @return the resource at the given path
     * @throws an
     *             exception if the resource is not unique under the given path
     */
    public Resource get(Session session, String path) throws DAOException {
        Resource result = null;
        logger.debug("Get resource from " + path);
        final String query = "from Resource res WHERE res.path = :path";
        try {
            Query q = session.createQuery(query);
            q.setParameter("path", path);
            result = (Resource) q.uniqueResult();

        } catch (HibernateException e) {
            throw new DAOException(e.getMessage());
        }
        return result;
    }

    /**
     * Delete a single resource under the given path
     * 
     * @param session
     * @param path
     * @return
     * @throws DAOException
     */
    public boolean delete(Session session, String path) throws DAOException {
        boolean result = true;

        if (logger.isDebugEnabled()) {
            logger.debug("Delete the resource from " + path);
        }

        if ((path == null) || (path.trim().length() == 0)) {
            if (logger.isDebugEnabled()) {
                logger.debug("Can not delete an entry under null or empty path");
            }
            result = false;
        } else {
            if (path.endsWith("/")) {
                this.deletePath(session, path);
            } else {
                this.deleteEntry(session, path);
            }
        }
        return result;
    }

    public int deletePath(final Session session, final String path) throws DAOException {
        int result = 0;
        final String query = "delete from Resource res WHERE res.path like '" + path + "%'";
        Query q = session.createQuery(query);
        logger.debug("Query =" + query);
        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            result = q.executeUpdate();
            tx.commit();
            logger.debug(result + " entries have been deleted");
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }
        return result;
    }

    public boolean deleteEntry(final Session session, final String path) throws DAOException {
        boolean result = true;
        logger.debug("Delete the entry from " + path);

        // get then delete...
        Transaction tx = null;
        try {

            tx = session.beginTransaction();
            Resource r = this.get(session, path);
            if (r != null) {
                session.delete(r);
                session.flush();
            }
            tx.commit();
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }
        return result;
    }

    public int deleteAll(Session session) throws DAOException {
        int result = 0;

        final String query = "delete from Resource";
        logger.debug("Query =" + query);
        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            Query q = session.createQuery(query);
            result = q.executeUpdate();
            tx.commit();
            logger.debug(result + " entries have been deleted");
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }
        return result;
    }

    public int deleteFrom(Session session, String registry) throws DAOException {
        int result = 0;

        final String query = "delete from Resource where rootRegistry = :rootRegistry";
        Query q = session.createQuery(query);
        q.setString("rootRegistry", registry);
        logger.debug("Query =" + query);
        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            result = q.executeUpdate();
            tx.commit();
            logger.debug(result + " entries have been deleted");
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }
        return result;
    }

    /**
     * Keep only the resources from the given registry
     * 
     * @param session
     * @param registry
     * @return
     * @throws DAOException
     */
    public int keepOnly(Session session, String registry) throws DAOException {
        int result = 0;

        final String query = "delete from Resource where rootRegistry != :rootRegistry";
        Query q = session.createQuery(query);
        q.setString("rootRegistry", registry);

        if (logger.isDebugEnabled()) {
            logger.debug("Query = '" + query + "' and rootregistry is " + registry);
        }

        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            result = q.executeUpdate();
            tx.commit();
            if (logger.isDebugEnabled()) {
                logger.debug(result + " entries have been deleted");
            }
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }
        return result;
    }

    public void update(Session session, Resource resource, String path) throws DAOException {
        // get the id if it exists
        // TODO : Thing about transaction...
        String id = this.getId(session, path);
        if (id != null) {
            resource.setId(id);
            this.merge(session, resource);
        } else {
            throw new DAOException("Can not find intial resource to update");
        }

    }

    public String getId(Session session, String path) {
        String result = null;
        String query = "from Resource where path = :path";
        Query q = session.createQuery(query);
        q.setString("path", path);

        Resource r = (Resource) q.uniqueResult();
        if (r != null) {
            result = r.getId();
        }
        return result;
    }

}
