/*
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.registry.core.transport.cxf;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import org.ow2.petals.registry.api.Information;
import org.ow2.petals.registry.api.Query;
import org.ow2.petals.registry.api.Resource;
import org.ow2.petals.registry.api.exception.RegistryException;
import org.ow2.petals.registry.api.transport.MessageSender;
import org.ow2.petals.registry.api.ws.Constants;
import org.ow2.petals.registry.api.ws.RemoteRegistryException;
import org.ow2.petals.registry.api.ws.service.RegistryService;
import org.ow2.petals.registry.core.ws.adapters.AdaptersManager;

/**
 * @author Christophe HAMERLING - eBM WebSourcing
 * @author Charles CASADEI - eBM WebSourcing
 */
public class CXFMessageSender implements MessageSender {

    private final ClientManager manager;

    private URI uri;

    public CXFMessageSender() {
        this.manager = new ClientManager();
    }

    protected RegistryService getRegistryClient(final String address) {
        return this.manager.getRegistryClient(address);
    }

    protected RegistryService getRegistryClient(final String address, final long timeout) {
        return this.manager.getRegistryClient(address, timeout);
    }

    protected String getAddress() {
        return this.uri.toString() + Constants.REGISTRY_SERVICE;
    }

    public String getType() {
        return "CXF";
    }

    public URI getURI() {
        return this.uri;
    }

    public void setURI(URI uri) {
        this.uri = uri;
    }

    public boolean delete(String path, Information information) throws RegistryException {
        RegistryService client = this.getRegistryClient(this.getAddress());
        if (client == null) {
            throw new RegistryException("Can not get the registry client for " + this.getAddress());
        }
        try {
            org.ow2.petals.registry.api.ws.to.Information i = AdaptersManager
                    .getInformationAdapter().toWS(information);
            return client.delete(path, i);
        } catch (RemoteRegistryException e) {
            throw new RegistryException(e.getMessage());
        } catch (Throwable e) {
            throw new RegistryException(e.getMessage());
        }
    }

    public Resource get(String path, Information information) throws RegistryException {
        Resource result = null;
        RegistryService client = this.getRegistryClient(this.getAddress());
        if (client == null) {
            throw new RegistryException("Can not get the registry client for " + this.getAddress());
        }
        try {
            org.ow2.petals.registry.api.ws.to.Information i = AdaptersManager
                    .getInformationAdapter().toWS(information);
            org.ow2.petals.registry.api.ws.to.Resource r = client.get(path, i);
            if (r != null) {
                result = AdaptersManager.getResourceAdapter().toRegistryResource(r);
            } else {
                // no result, throw exception?
            }
        } catch (RemoteRegistryException e) {
            throw new RegistryException(e.getMessage());
        } catch (Throwable e) {
            throw new RegistryException(e.getMessage());
        }

        return result;
    }

    public boolean put(String path, Resource resource, Information information)
            throws RegistryException {
        RegistryService client = this.getRegistryClient(this.getAddress());
        if (client == null) {
            throw new RegistryException("Can not get the registry client for " + this.getAddress());
        }
        try {
            org.ow2.petals.registry.api.ws.to.Resource r = AdaptersManager.getResourceAdapter()
                    .toWSResource(resource);
            org.ow2.petals.registry.api.ws.to.Information i = AdaptersManager
                    .getInformationAdapter().toWS(information);
            return client.put(path, r, i);
        } catch (RemoteRegistryException e) {
            throw new RegistryException(e.getMessage());
        } catch (Throwable e) {
            throw new RegistryException(e.getMessage());
        }

    }

    public List<Resource> getAll(String path, Information information) throws RegistryException {
        List<Resource> result = new ArrayList<Resource>();
        RegistryService client = this.getRegistryClient(this.getAddress());
        if (client == null) {
            throw new RegistryException("Can not get the registry client for " + this.getAddress());
        }
        try {
            org.ow2.petals.registry.api.ws.to.Information i = AdaptersManager
                    .getInformationAdapter().toWS(information);
            List<org.ow2.petals.registry.api.ws.to.Resource> r = client.getAll(path, i);

            if (r != null) {
                for (org.ow2.petals.registry.api.ws.to.Resource resource : r) {
                    result.add(AdaptersManager.getResourceAdapter().toRegistryResource(resource));
                }
            }

        } catch (RemoteRegistryException e) {
            throw new RegistryException(e.getMessage());
        } catch (Throwable e) {
            throw new RegistryException(e.getMessage());
        }

        return result;
    }

    public List<Resource> query(Query query, Information info) throws RegistryException {
        List<Resource> result = new ArrayList<Resource>();
        RegistryService client = this.getRegistryClient(this.getAddress());
        if (client == null) {
            throw new RegistryException("Can not get the registry client for " + this.getAddress());
        }
        try {
            org.ow2.petals.registry.api.ws.to.Information i = AdaptersManager
                    .getInformationAdapter().toWS(info);
            org.ow2.petals.registry.api.ws.to.Query q = AdaptersManager.getQueryAdapter().toWS(
                    query);
            List<org.ow2.petals.registry.api.ws.to.Resource> r = client.query(q, i);

            if (r != null) {
                for (org.ow2.petals.registry.api.ws.to.Resource resource : r) {
                    result.add(AdaptersManager.getResourceAdapter().toRegistryResource(resource));
                }
            }

        } catch (RemoteRegistryException e) {
            throw new RegistryException(e.getMessage());
        } catch (Throwable e) {
            throw new RegistryException(e.getMessage());
        }

        return result;
    }
}
