/*
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.registry.core.ws;

import org.apache.cxf.endpoint.Server;
import org.apache.cxf.jaxws.JaxWsServerFactoryBean;
import org.ow2.petals.registry.api.LocalRegistry;
import org.ow2.petals.registry.api.context.Context;
import org.ow2.petals.registry.api.exception.LifeCycleException;
import org.ow2.petals.registry.api.lifecycle.AbstractLifeCycle;
import org.ow2.petals.registry.api.ws.service.RegistryClientService;
import org.ow2.petals.registry.core.manager.AbstractRemoteClientManager;
import org.ow2.petals.registry.core.ws.service.RegistryClientServiceImpl;

/**
 * Expose the local registry as a web service for clients.
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 */
public class WSRemoteClientManager extends AbstractRemoteClientManager {

    /**
     * The CXF server
     */
    private Server server;

    /**
     * The service address
     */
    private String address;

    public WSRemoteClientManager() {
        this.lifeCycle = new AbstractLifeCycle() {
            @Override
            protected void doInit(Context context) throws LifeCycleException {
                address = buildAddress(context.getConfiguration().getClientManagerURL());
            }

            @Override
            protected void doStart() throws LifeCycleException {
                logger.info("Starting remote client manager web service on " + address);
                JaxWsServerFactoryBean svrFactory = new JaxWsServerFactoryBean();
                RegistryClientService service = new RegistryClientServiceImpl(getLocalRegistry());
                svrFactory.setAddress(address);
                svrFactory.setServiceBean(service);
                server = svrFactory.create();
            }

            @Override
            protected void doStop() throws LifeCycleException {
                server.stop();
            }
        };
    }

    public LocalRegistry getLocalRegistry() {
        return this.localRegistry;
    }

    public void setLocalRegistry(LocalRegistry localRegistry) {
        this.localRegistry = localRegistry;
    }

    private String buildAddress(String rootURL) {
        return rootURL + org.ow2.petals.registry.api.ws.Constants.REMOTE_CLIENT_SERVICE;
    }
}
