/*
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.registry.core.ws.service;

import java.util.ArrayList;
import java.util.List;

import javax.jws.WebService;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.petals.registry.api.exception.RegistryException;
import org.ow2.petals.registry.api.manager.IncomingManager;
import org.ow2.petals.registry.api.ws.RemoteRegistryException;
import org.ow2.petals.registry.api.ws.service.RegistryService;
import org.ow2.petals.registry.api.ws.to.Information;
import org.ow2.petals.registry.api.ws.to.Query;
import org.ow2.petals.registry.api.ws.to.Resource;
import org.ow2.petals.registry.core.ws.adapters.AdaptersManager;

/**
 * This web service is used by the registry to expose its message receiver as a
 * web service. It is used by registries to communicate together (not intended
 * for client usage)..
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 */
@WebService
public class RegistryServiceImpl implements RegistryService {

    private final Log logger = LogFactory.getLog(RegistryServiceImpl.class);

    /**
     * Use the local manager since it is it role to choose what to do with the
     * received data
     */
    private final IncomingManager localManager;

    public RegistryServiceImpl(final IncomingManager localManager) {
        this.localManager = localManager;
    }

    public boolean delete(String path, Information information) throws RemoteRegistryException {
        this.logger.debug("Receiving a remote call to delete resource from path " + path);
        try {
            return this.localManager.delete(path, AdaptersManager.getInformationAdapter()
                    .toRegistry(information));
        } catch (RegistryException e) {
            throw new RemoteRegistryException(e.getMessage());
        }
    }

    public Resource get(String path, Information information) throws RemoteRegistryException {
        this.logger.debug("Receiving a remote call to Get resource from" + path);
        Resource result = null;
        org.ow2.petals.registry.api.Resource r = null;
        try {
            r = this.localManager.get(path, AdaptersManager.getInformationAdapter().toRegistry(
                    information));
        } catch (RegistryException e) {
            throw new RemoteRegistryException(e.getMessage());
        }
        if (r != null) {
            result = AdaptersManager.getResourceAdapter().toWSResource(r);
        }
        return result;
    }

    public List<Resource> getAll(String path, Information information)
            throws RemoteRegistryException {
        this.logger.debug("Receiving a remote call to GetAll from " + path);
        List<Resource> result = null;
        List<org.ow2.petals.registry.api.Resource> r = null;
        try {
            r = this.localManager.getAll(path, AdaptersManager.getInformationAdapter().toRegistry(
                    information));
        } catch (RegistryException e) {
            throw new RemoteRegistryException(e.getMessage());
        }
        if (r != null) {
            result = new ArrayList<Resource>(r.size());
            for (org.ow2.petals.registry.api.Resource resource : r) {
                result.add(AdaptersManager.getResourceAdapter().toWSResource(resource));
            }
        }
        return result;
    }

    public boolean put(String path, Resource resource, Information information)
            throws RemoteRegistryException {
        this.logger.debug("Receiving a remote call to Put resource to path " + path);
        org.ow2.petals.registry.api.Resource r = AdaptersManager.getResourceAdapter()
                .toRegistryResource(resource);
        try {
            return this.localManager.put(path, r, AdaptersManager.getInformationAdapter()
                    .toRegistry(information));
        } catch (RegistryException e) {
            throw new RemoteRegistryException(e.getMessage());
        }
    }

    public List<Resource> query(Query query, Information information)
            throws RemoteRegistryException {
        this.logger.debug("Receiving a remote call to Query with " + query);
        List<Resource> result = null;
        List<org.ow2.petals.registry.api.Resource> r = null;
        try {
            r = this.localManager.query(AdaptersManager.getQueryAdapter().toRegistry(query),
                    AdaptersManager.getInformationAdapter().toRegistry(information));
        } catch (RegistryException e) {
            throw new RemoteRegistryException(e.getMessage());
        }
        if (r != null) {
            result = new ArrayList<Resource>(r.size());
            for (org.ow2.petals.registry.api.Resource resource : r) {
                result.add(AdaptersManager.getResourceAdapter().toWSResource(resource));
            }
        }
        return result;
    }

}
